package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"gas-cylinder-api/common/global"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type CustomerController struct {
	api.Api
}

// GetPage 获取客户信息列表
// @Summary 获取客户信息列表
// @Description 获取客户信息列表
// @Tags 客户信息
// @Param principalPhone query string false "电话"
// @Param type           query string false "类型"
// @Param address        query string false "地址"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.Customer}} "{"code": 200, "data": [...]}"
// @Router /api/customer [get]
// @Security Bearer
func (e CustomerController) GetPage(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	list := make([]model.Customer, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取客户信息
// @Summary 通过id获取客户信息
// @Description 通过id获取客户信息
// @Tags 客户信息
// @Param id path string true "客户信息id"
// @Success 200 {object} response.Response{data=model.Customer} "{"code": 200, "data": [...]}"
// @Router /api/customer/{id} [get]
// @Security Bearer
func (e CustomerController) Get(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.Customer

	//数据权限检查
	err = s.Get(&req, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// GetByPhone 通过手机号获取客户信息
// @Summary 通过手机号获取客户信息
// @Description 通过手机号获取客户信息
// @Tags 客户信息
// @Param phone query string true "手机号"
// @Success 200 {object} response.Response{data=model.Customer} "{"code": 200, "data": [...]}"
// @Router /api/customer/phone [get]
// @Security Bearer
func (e CustomerController) GetByPhone(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerGetByPhoneReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.Customer
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.GetByPhone(&req, &object, p)
	if err != nil {
		if err == global.GetNotFoundErr {
			e.Error(404, err, err.Error())
			return
		} else {
			e.Error(500, err, err.Error())
			return
		}
	}

	e.OK(object, "查询成功")
}

// Insert 添加客户信息
// @Summary 添加客户信息
// @Description 添加客户信息
// @Tags 客户信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CustomerInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/customer [post]
// @Security Bearer
func (e CustomerController) Insert(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	// 获取销售门店code
	store, err := model.GetProvCodeById(p.DeptId)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	req.SetStoreCode(store.CmpCode)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改客户信息
// @Summary 修改客户信息
// @Description 修改客户信息
// @Tags 客户信息
// @Accept  application/json
// @Product application/json
// @Param id path string true "客户信息id"
// @Param data body dto.CustomerUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/customer [put]
// @Security Bearer
func (e CustomerController) Update(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除客户信息
// @Summary 删除客户信息
// @Description 删除客户信息
// @Tags 客户信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CustomerDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/customer [delete]
// @Security Bearer
func (e CustomerController) Delete(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}

// InsertOrUpdate 添加客户或更新信息
// @Summary 添加客户或更新信息
// @Description 添加客户或更新信息
// @Tags 客户信息
// @Accept  application/json
// @Product application/json
// @Param data body dto.CustomerInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/customer [post]
// @Security Bearer
func (e CustomerController) InsertOrUpdate(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	// 获取销售门店code
	store, err := model.GetProvCodeById(p.DeptId)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	req.SetStoreCode(store.CmpCode)
	// 通过手机号查询客户信息
	err = s.InsertOrUpdate(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "提交成功")
}

// GetBorrowGasCylinder 获取在用气瓶
// @Summary 获取在用气瓶
// @Description 获取在用气瓶
// @Tags 客户信息
// @Param phone query string true "手机号"
// @Success 200 {object} response.Response{data=model.Customer} "{"code": 200, "data": [...]}"
// @Router /api/customer/phone [get]
// @Security Bearer
func (e CustomerController) GetBorrowGasCylinder(c *gin.Context) {
	s := service.Customer{}
	req := dto.CustomerGetBorrowGasCylinderReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	list := make([]model.CustomerGasCylinder, 0)

	//数据权限检查
	err = s.GetBorrowGasCylinder(&req, &list)
	if err != nil {
		if err == global.GetNotFoundErr {
			e.Error(404, err, err.Error())
			return
		} else {
			e.Error(500, err, err.Error())
			return
		}
	}

	e.PageOK(list, 0, 0, 0, "查询成功")
}
