package controller

import (
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	"gas-cylinder-api/common/global"
	"github.com/gin-gonic/gin"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
	"gogs.baozhida.cn/zoie/OAuth-core/sdk"
	"gogs.baozhida.cn/zoie/OAuth-core/sdk/config"
	"net/http"

	"github.com/gin-gonic/gin/binding"
)

type SysRoleController struct {
	api.Api
}

// GetPage 获取角色列表
// @Summary 角色列表数据
// @Description 获取角色列表
// @Tags 角色
// @Param roleName query string false "角色名称"
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysRole}} "{"code": 200, "data": [...]}"
// @Router /api/role [get]
// @Security Bearer
func (e SysRoleController) GetPage(c *gin.Context) {
	s := service.SysRole{}
	req := dto.SysRoleGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Form).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	list := make([]model.SysRole, 0)
	var count int64
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取角色
// @Summary 通过id获取角色
// @Description 通过id获取角色
// @Tags 角色
// @Param id path string true "角色Id"
// @Success 200 {object} response.Response{data=model.SysRole} "{"code": 200, "data": [...]}"
// @Router /api/role/{id} [get]
// @Security Bearer
func (e SysRoleController) Get(c *gin.Context) {
	s := service.SysRole{}
	req := dto.SysRoleGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var object model.SysRole
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Logger.Error(err)
		e.Error(http.StatusUnprocessableEntity, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 创建角色
// @Summary 创建角色
// @Description 创建角色
// @Tags 角色
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysRoleInsertReq true "data"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/role [post]
// @Security Bearer
func (e SysRoleController) Insert(c *gin.Context) {
	s := service.SysRole{}
	req := dto.SysRoleInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)

	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	_, err = global.LoadPolicy(c)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改用户角色
// @Summary 修改用户角色
// @Description 修改用户角色
// @Tags 角色
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysRoleUpdateReq true "body"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/role/{id} [put]
// @Security Bearer
func (e SysRoleController) Update(c *gin.Context) {
	s := service.SysRole{}
	req := dto.SysRoleUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Update(&req, p)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//_, err = global.LoadPolicy(c)
	//if err != nil {
	//	e.Logger.Error(err)
	//	e.Error(500, err, err.Error())
	//	return
	//}

	e.OK(req.GetId(), "更新成功")
}

// Delete 删除用户角色
// @Summary 删除用户角色
// @Description 删除用户角色
// @Tags 角色
// @Accept  application/json
// @Product application/json
// @Param data body dto.SysRoleDeleteReq true "body"
// @Success 200 {object} response.Response "{"code": 200, "data": [...]}"
// @Router /api/role [delete]
// @Security Bearer
func (e SysRoleController) Delete(c *gin.Context) {
	s := new(service.SysRole)
	req := dto.SysRoleDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	userReq := dto.SysUserGetCountReq{RoleIds: []int{req.Id}}
	var count int64
	err = userSvc.GetCount(&userReq, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	if count > 0 {
		e.Error(500, err, "有用户关联，禁止删除！")
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	//cb := sdk.Runtime.GetCasbinKey(c.Request.Host)
	cb := sdk.Runtime.GetCasbinKey(config.ApplicationConfig.Host)
	// fixme 删除角色与路由，角色与菜单之间的关联
	err = s.Remove(&req, p, cb)
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	e.OK(req.GetId(), "删除成功")
}
