package service

import (
	"errors"
	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service/dto"
	"gas-cylinder-api/common/actions"
	cDto "gas-cylinder-api/common/dto"
	"gas-cylinder-api/common/global"
	"gogs.baozhida.cn/zoie/OAuth-core/service"
	"gorm.io/gorm"
)

type GasCylinderSpec struct {
	service.Service
}

// GetPage 获取GasCylinderSpec列表
func (e *GasCylinderSpec) GetPage(c *dto.GasCylinderSpecGetPageReq, list *[]model.GasCylinderSpec, count *int64, p *actions.DataPermission) error {
	var err error
	var data model.GasCylinderSpec

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			actions.Permission(data.TableName(), p),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}

// Get 获取GasCylinderSpec对象
func (e *GasCylinderSpec) Get(d *dto.GasCylinderSpecGetReq, gasCylinderAllotModel *model.GasCylinderSpec, p *actions.DataPermission) error {
	err := e.Orm.
		Scopes(actions.Permission(gasCylinderAllotModel.TableName(), p)).
		First(gasCylinderAllotModel, d.GetId()).Error

	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}

	return nil
}

// Insert 创建GasCylinderSpec对象
func (e *GasCylinderSpec) Insert(c *dto.GasCylinderSpecInsertReq) error {
	var err error
	var data model.GasCylinderSpec

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	// 添加角色
	c.Generate(&data)
	err = tx.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	c.Id = data.Id
	return nil

}

// Update 修改GasCylinderSpec对象
func (e *GasCylinderSpec) Update(c *dto.GasCylinderSpecUpdateReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var gasCylinderAllotModel = model.GasCylinderSpec{}
	// 查询角色是否存在
	err = e.Orm.Scopes(actions.Permission(gasCylinderAllotModel.TableName(), p)).
		First(&gasCylinderAllotModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	c.Generate(&gasCylinderAllotModel)
	err = tx.Save(&gasCylinderAllotModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = gasCylinderAllotModel.Id

	return nil
}

// Remove 删除GasCylinderSpec
func (e *GasCylinderSpec) Remove(c *dto.GasCylinderSpecDeleteReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var gasCylinderAllotModel model.GasCylinderSpec

	// 查询角色是否存在
	err = e.Orm.Scopes(actions.Permission(gasCylinderAllotModel.TableName(), p)).
		First(&gasCylinderAllotModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.DeleteNotFoundOrNoPermissionErr
		}
		return global.DeleteFailedErr
	}

	db := tx.Delete(&gasCylinderAllotModel)

	if err = db.Error; err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.DeleteFailedErr
	}
	if db.RowsAffected == 0 {
		return global.DeleteNotFoundOrNoPermissionErr
	}

	return nil
}

// GetAppletPage 获取GasCylinderSpec列表
func (e *GasCylinderSpec) GetAppletPage(c *dto.AppletGasCylinderSpecGetPageReq, list *[]model.GasCylinderSpec, count *int64) error {
	var err error
	var data model.GasCylinderSpec

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}
