package service

import (
	"encoding/json"
	"errors"
	"fmt"
	"gas-cylinder-api/common/actions"
	cDto "gas-cylinder-api/common/dto"
	"gas-cylinder-api/common/global"
	model2 "gas-cylinder-api/common/model"
	"gogs.baozhida.cn/zoie/OAuth-core/service"
	"math/rand"
	"time"

	"gas-cylinder-api/app/admin/model"
	"gas-cylinder-api/app/admin/service/dto"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg"
	"gorm.io/gorm"
)

type Store struct {
	service.Service
}

// 获取所有上级目录
func InDeptIdScopes(deptId int) func(db *gorm.DB) *gorm.DB {
	return func(db *gorm.DB) *gorm.DB {
		if deptId == 0 {
			return db
		}
		ids, err := model.GetParentIds(deptId)
		if err != nil {
			return db
		}
		return db.Where("dept_id in (?)", ids)
	}
}

// 获取顶级公司下所有子公司
func InRootCompanyIdsScopes(deptId int) func(db *gorm.DB) *gorm.DB {
	return func(db *gorm.DB) *gorm.DB {
		if deptId == 0 {
			return db
		}
		ids, err := model.GetRootCompanyIds(deptId)
		if err != nil {
			return db
		}
		return db.Where("dept_id in (?)", ids)
	}
}

// Get 获取SysDept对象
func (e *Store) Get(d *dto.StoreGetReq, deptModel *model.SysDept) error {
	err := e.Orm.
		First(deptModel, d.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}
	return nil
}

// Insert 创建SysDept对象
func (e *Store) Insert(c *dto.StoreInsertReq, deptId int) error {
	var err error
	var data model.SysDept
	tx := e.Orm.Debug().Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	// TODO 上传省平台-1.1.1.4新增销售门店信息 addStore
	//storeCode = res.StoreCode
	rand.New(rand.NewSource(time.Now().UnixNano()))
	// 生成随机数后八位
	randomSuffix := rand.Intn(9999999999999999)

	// 组合成16位随机数
	storeCode := fmt.Sprintf("%16d", randomSuffix)
	data.CmpCode = storeCode
	c.Generate(&data)
	data.UploadTime = model2.Time(time.Now())
	if deptId != 0 && c.ParentId == 0 {
		data.ParentId = deptId
	}
	err = tx.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}

	c.Id = data.Id
	deptPath := pkg.IntToString(data.Id) + "/"
	if data.ParentId != 0 {
		var deptP model.SysDept
		err = tx.First(&deptP, data.ParentId).Error
		if err != nil {
			e.Log.Errorf("db error: %s", err)
			return errors.New("获取父级信息失败")
		}
		deptPath = deptP.Path + deptPath
	} else {
		deptPath = "/0/" + deptPath
	}
	var mp = map[string]string{}
	mp["path"] = deptPath

	err = tx.Model(&data).Update("path", deptPath).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	return nil
}

// Update 修改SysDept对象
func (e *Store) Update(c *dto.StoreUpdateReq) error {
	var err error
	var deptModel = model.SysDept{}
	tx := e.Orm.Debug().Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	err = tx.First(&deptModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	c.Generate(&deptModel)

	DeptPath := pkg.IntToString(deptModel.Id) + "/"
	if deptModel.ParentId != 0 {
		var DeptP model.SysDept
		tx.First(&DeptP, deptModel.ParentId)
		DeptPath = DeptP.Path + DeptPath
	} else {
		DeptPath = "/0/" + DeptPath
	}
	deptModel.Path = DeptPath
	db := tx.Save(&deptModel)
	if err = db.Error; err != nil {
		e.Log.Errorf("UpdateSysDept error: %s", err)
		return err
	}
	return nil
}

// Remove 删除SysDept
// 删除部门 同时删除其子部门、部门下的用户
func (e *Store) Remove(d *dto.StoreDeleteReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()
	var dept model.SysDept
	err = e.Orm.
		First(&dept, d.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.DeleteFailedErr
	}

	var deptList = make([]model.SysDept, 0)
	var subDeptIds = make([]int, 0)
	err = tx.Where("path like ?", dept.Path+"%").Find(&deptList).Error
	if err != nil {
		e.Log.Errorf("Delete User error: %s", err)
		return errors.New("查询子门店失败！")
	}

	// 获取部门及其子部门下ID
	for _, v := range deptList {
		if v.Id != dept.Id {
			subDeptIds = append(subDeptIds, v.Id)
		}
	}

	if len(subDeptIds) > 0 {
		return errors.New("请先删除子门店！")
	}
	db := tx.Delete(&dept)
	if err = db.Error; err != nil {
		e.Log.Errorf("Delete error: %s", err)
		return global.DeleteFailedErr
	}

	return nil
}

// getList 获取组织数据
func (e Store) getList(c *dto.StoreGetPageReq, list *[]model.SysDept, p *actions.DataPermission) error {
	var err error
	var data model.SysDept
	if len(c.Name) == 0 && c.DeptId == 0 {
		err = e.Orm.Model(&data).
			Scopes(
				cDto.MakeCondition(c.GetNeedSearch()),
			).
			Find(list).Error
		if err != nil {
			e.Log.Errorf("db error:%s", err)
			return err
		}

		return nil
	}

	var tempList []model.SysDept
	err = e.Orm.Model(&data).Where("id in (select id from sys_dept where path like ? )", "%/"+pkg.IntToString(p.DeptId)+"/%").
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
		).
		Find(&tempList).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}

	// 获取部门及其子部门下ID
	for _, dept := range tempList {
		var deptList = make([]model.SysDept, 0)
		err = e.Orm.Where("path like ?", dept.Path+"%").Order(fmt.Sprintf("sort %s", c.SortOrder)).Find(&deptList).Error
		*list = append(*list, deptList...)
	}
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}

	*list = DeduplicateStore(*list)

	return nil
}

func StoreDistrictScopes(area string) func(db *gorm.DB) *gorm.DB {

	return func(db *gorm.DB) *gorm.DB {
		if len(area) == 0 {
			return db
		}
		return db.Where("JSON_EXTRACT(prov_store, '$.district') = ? ", area)
	}
}

func StoreCityScopes(city string) func(db *gorm.DB) *gorm.DB {

	return func(db *gorm.DB) *gorm.DB {
		if len(city) == 0 {
			return db
		}
		return db.Where("JSON_EXTRACT(prov_store, '$.city') = ? ", city)
	}
}
func (e Store) getAppletList(c *dto.AppletStoreGetPageReq, list *[]model.SysDept) error {
	var err error
	var data model.SysDept

	var tempList []model.SysDept
	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			StoreDistrictScopes(c.District),
			StoreCityScopes(c.City),
		).
		Find(&tempList).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}

	// 获取部门及其子部门下ID
	for _, dept := range tempList {
		var deptList = make([]model.SysDept, 0)
		err = e.Orm.Where("path like ?", dept.Path+"%").Order(fmt.Sprintf("sort %s", c.SortOrder)).Find(&deptList).Error
		*list = append(*list, deptList...)
	}
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}

	*list = DeduplicateStore(*list)

	return nil
}

// SetStorePage 设置Dept页面数据
func (e *Store) SetStorePage(c *dto.StoreGetPageReq, p *actions.DataPermission) (m []model.SysDept, err error) {
	var list []model.SysDept
	err = e.getList(c, &list, p)
	for i := 0; i < len(list); i++ {
		if list[i].ParentId != 0 {
			continue
		}
		info := e.StorePageCall(&list, list[i])
		m = append(m, info)
	}

	if len(m) == 0 {
		parentMap, flag := getStoreParent(list)
		if flag {
			for i := 0; i < len(list); i++ {
				if parentMap[list[i].ParentId] {
					continue
				}
				info := e.StorePageCall(&list, list[i])
				m = append(m, info)
			}
		} else {
			for i := 0; i < len(list); i++ {
				info := e.StorePageCall(&list, list[i])
				m = append(m, info)
			}
		}
	}

	return
}
func (e *Store) SetAppletStorePage(c *dto.AppletStoreGetPageReq) (m []model.SysDept, err error) {
	var list []model.SysDept
	err = e.getAppletList(c, &list)
	for i := 0; i < len(list); i++ {
		if list[i].ParentId != 0 {
			continue
		}
		info := e.StorePageCall(&list, list[i])
		m = append(m, info)
	}

	if len(m) == 0 {
		parentMap, flag := getStoreParent(list)
		if flag {
			for i := 0; i < len(list); i++ {
				if parentMap[list[i].ParentId] {
					continue
				}
				info := e.StorePageCall(&list, list[i])
				m = append(m, info)
			}
		} else {
			for i := 0; i < len(list); i++ {
				info := e.StorePageCall(&list, list[i])
				m = append(m, info)
			}
		}
	}

	return
}

func getStoreParent(deptList []model.SysDept) (map[int]bool, bool) {
	list := deptList
	var flag = false
	var parentMap = map[int]bool{}
	for j := 0; j < len(list); j++ {
		parentMap[list[j].ParentId] = false
	}

	for j := 0; j < len(list); j++ {
		if _, ok := parentMap[list[j].Id]; !ok {
			continue
		}
		parentMap[list[j].Id] = true
		flag = true
	}
	return parentMap, flag
}

func (e *Store) StorePageCall(deptList *[]model.SysDept, menu model.SysDept) model.SysDept {
	list := *deptList
	min := make([]model.SysDept, 0)
	for j := 0; j < len(list); j++ {
		if menu.Id != list[j].ParentId {
			continue
		}
		mi := model.SysDept{}
		mi.Id = list[j].Id
		mi.ParentId = list[j].ParentId
		mi.Path = list[j].Path
		mi.Name = list[j].Name
		mi.Sort = list[j].Sort
		mi.Status = list[j].Status
		mi.CreatedAt = list[j].CreatedAt
		mi.ProvStore = list[j].ProvStore
		mi.CmpCode = list[j].CmpCode
		mi.Children = []model.SysDept{}
		ms := e.StorePageCall(deptList, mi)
		min = append(min, ms)
	}
	menu.Children = min
	return menu
}

func DeduplicateStore(list []model.SysDept) []model.SysDept {
	seen := make(map[string]bool)
	deduplicated := []model.SysDept{}

	for _, v := range list {
		// 将结构体编码为 JSON 字符串
		key, err := json.Marshal(v)
		if err != nil {
			// 处理错误
			continue
		}
		// 将 JSON 字符串作为键插入到 map 中
		if _, ok := seen[string(key)]; !ok {
			seen[string(key)] = true
			deduplicated = append(deduplicated, v)
		}
	}

	return deduplicated
}
